;*******************************************************************************
;Pracovni frekvence krystalu je 18432kHz => frekvence CPU je 2048kHz
;Prenosova rychlost je 19140bd (19200bd), prenasi se 1 START BIT+8 BITU+PARITA
;-------------------------------------------------------------------------------
;Zapojeni konektoru DIN5:	1 - RS232 SERIAL OUT
;(cislovani podle konektoru)	2 - GND
;				3 - RS232 SERIAL IN
;				4 - Vin_1 - symetricke, stridave napeti proti GND
;				5 - Vin_2 - symetricke, stridave napeti proti GND
;*******************************************************************************
;Dekoder adres MH3205 (IC11):
;A - /DBIN	G1	- /DBIN NAND /WR
;B - A14	/G2A	- /Q 7474 (IC8)
;C - A15	/G2B	- /Q 7474
;-------------------------------------------------------------------------------
;Y0 - READ EPROM	- cteni EPROM, adresovy rozsah 0000-07FF	RD 0000h
;Y1 - WRITE DISPLAY	- viz. nize					WR 0000h
;Y2 - READ SRAM		- cteni RAM, adresovy rozsah 4000-43FF		RD 4000h
;Y3 - WRITE SRAM	- zapis RAM, adresovy rozsah 4000-43FF		WR 4000h
;Y4 - READ KBD+SERIAL	- viz. nize					RD 8000h
;Y5 - WRITE SERIAL	- vystup D7 pres MH7474	(IC8)			WR 8000h
;Y6 - nepouzit		- nezapojeno					RD C000h
;Y7 - WRITE DISPLAY 	- viz. nize					WR C000h
;-------------------------------------------------------------------------------
;Y1 - WR 0000	- zapis D0..D5 do 74174 - D0..D2 - dekoder MH3205 - radky klavesnice
;								  - piny R1..R7 displeje
;					- D3,D4	 - nepouzito
;					- D5	 - povoluje MH3205 (IC16),
;                                                  generuje puls 74123 (IC17)
;		  Y7=1 - na vstupech 7442 je 1xxx - dekoduje vystup 8 a 9 - nepouzite
;-------------------------------------------------------------------------------
;Y4 - RD 8000	- cteni D0..D4+D7 z 4503 - D0..D4 - sloupce klavesnice
;				  (IC12) - D7	 - vstup seriovych dat, spojeno s INT 8080A
;-------------------------------------------------------------------------------
;Y7 - WR C00x	- zapis D0..D5 do 74174	- D0..D4 - zobrazovana data SL1..SL5
;			 	  (IC6)	  D5=0   - blokovani 2. MH3205 (IC16)
;		- A0..A2 + Y7=0			 - dekoder 7442 displeje, pozice 0..7
;*******************************************************************************
;Dekoder displeje a klavesnice MH3205 (IC16):
;A - /D0	G1	- D5
;B - D1		/G2A	- /Q 74123 (IC17)
;C - D2		/G2B	- GND
;-------------------------------------------------------------------------------
;	display		klavesnice	
;Y0 -	--		KL1		v klidovem stavu jsou vsechny Y0..Y7
;Y1 -	RA1		KL2		a D0..D4 ve stavu '1'. MH3205 stahne
;Y2 - 	RA3		KL3		Yn do stavu '0' a v pripade stisku klavesy
;Y3 - 	RA2		KL4_11_12	se stav '0' promitne i na vstupni Dx
;Y4 -	RA6		KL5		
;Y5 -	RA7		KL13		
;Y6 -	RA4		KL14		
;Y7 -	RA5		KL15		
;*******************************************************************************
;Organizace VRAM: 1. znak	8.znak		zobrazeni bitu 01234xxx
;	1.uradek  04208h........0420Fh		
;	2.uradek  04210h........04217h
;	3.uradek  04218h........0421Fh
;	4.uradek  04220h........04227h
;	5.uradek  04228h........0422Fh
;	6.uradek  04230h........04237h
;	7.uradek  04238h........0423Fh
;*******************************************************************************
;Prijimaci buffer seriove linky ma 256 byte a zacina na adrese 4100 - meni
;se pouze nizsi byte registroveho paru 
;*******************************************************************************

RAM_ST: EQU     04000h          ;pocatek RAM 2114
RAM_TP: EQU     043FFh          ;konec RAM 2114
RAM_VR: EQU     04250h          ;pocatek promennych v RAM
SERIAL: EQU     08000h          ;adresa pro zapis/cteni seriovych dat a displeje
DISP:   EQU     0C000h          ;vystupni adresa pro zapis do displeje

	ORG	00000h
L0000:  LD	SP,0424Ch	;oblast promennych
        LD	HL,04000h
        PUSH	HL		;V424A - adresa znakove VRAM pro zobrazeni
        PUSH	HL		;V4248 - 		-"-
        PUSH	HL		;V4246 - 		-"-
        LD	HL,0423Fh
        PUSH	HL		;V4244 - posledni byte graficke VRAM
        LD	HL,04100h
        PUSH	HL		;V4242 - adresa v bufferu pro cteni prijatych dat
        PUSH	HL		;V4240 - adresa v bufferu pro zapis prijatych dat
        LD	SP,RAM_TP	;vrchol zasobniku
        LD	HL,04250h	;kam se ukada Acc (4250h az 4260) - nulovani promennych
        SUB	A		;nuluj Acc
        CALL	L0001		;ulozeni Acc na (HL) - 16x
        LD	HL,04200h	;kam se uklada Acc (4200h az 4210)
        CALL	L0001		;ulozeni Acc na (HL) - 16x
        LD	HL,04000h	;kam se uklada Acc (4000h az 4010) - ASCII znaky pro zobrazeni
        LD	A,020h		;v Acc je 020h - ASCII ' '
        CALL	L0001		;ulozeni Acc na (HL) - 16x - smazani displeje
        CALL	L0002		;naplneni VRAM bitmapami jednotlivych znaku
        JP	L0003		;hlavni programova smycka

        DB	0FFh,0FFh,0FFh,0FFh,0FFh,0FFh,0FFh,0FFh

;*******************************************************************************
;Obsluha INT
;*******************************************************************************
LRSTF:	PUSH	AF		;schovej si registry
        PUSH	BC
        LD	A,A
        LD	A,(SERIAL)	;cti vstupni byte
        RLCA			;D7 do CY
        JP	NC,L0004	;CY=0 - konec preruseni
        PUSH	DE		;schovej si registr 
        LD	BC,00008h	;priprav promenne
L0006:  LD	DE,00001h	;delka casove prodlevy
        CALL	L0005		;casova prodleva s delkou v DE
        LD	A,(SERIAL)	;cti vstupni byte
        RLA			;D7 do CY
        LD	A,B		;vystupni byte
        RRA			;CY do D0
        LD	B,A		;vystupni byte zpatky
        DEC	C		;citac pruchodu
        JP	NZ,L0006	;cti vsech 8 bitu
        CPL			;zneguj prectena data
        LD	B,A		;presun do B - prijaty byte
        OR	A		;nastaveni FLAGS
        LD	A,080h		;prednastaveni sude parity prectenaho byte
        JP	PE,L0007	;je suda parita, skoc
        SUB	A		;parita byla licha - nastav
L0007:  LD	C,A		;schovej si priznak parity
        PUSH	BC		;casova prodleva
        POP	BC
        LD	A,(SERIAL)	;cti vstupni byte
        AND	080h		;ponechej jen D7 - parita preneseneho byte
        CP	C		;porovnej s ocekavanym vysledkem
        JP	NZ,L0008	;parita se neshoduje, skoc
        CALL	L0009
        LD	A,B		;prijaty byte
        RLCA			;D7 do CY
        JP	NC,L0010
        LD	A,(SERIAL)	;cti seriovou linku
        RLCA			;D7 do CY
        JP	C,L0011		;nastavi V425B na 080h, konec preruseni
        LD	A,(V05FF)	;zde je 093h - cislo terminalu?
        CP	B		;porovnej prijaty byte a byte z (005FFh)
        JP	Z,L0012		;je shoda - skoc
        OR	040h		;nastav v Acc bit D6 - 0D3h
        CP	B		;porovnej s prijatym byte
        JP	Z,L0013		;je shoda - skoc
        INC	B		;
        JP	Z,L0013
        SUB	A		;nuluj Acc - prijaty byte neni ocekavan
L0013:  LD	(V4250),A
L0017:  POP	DE		;obnov registry
L0004:  POP	BC
        POP	AF
        EI			;povol preruseni
        RET			;navrat

L0012:  SUB	A		;nuluj Acc
        LD	(SERIAL),A	;shod seriovou linku - zacatek vysilani
        LD	A,(V4251)
        LD	(V425E),A
        LD	C,A
        LD	A,(V425B)
        OR	C
        LD	C,A
        SUB	A		;nuluj Acc
        LD	(V425B),A
        LD	B,008h		;delka vysilaneho byte je 8 bitu
        JP	L0014		;preskoc casovou prodlevu

L0015:  LD	DE,00001h	;delka casove prodlevy
        CALL	L0005		;casova prodleva s delkou v DE
L0014:  LD	A,C		;vysilany byte
        RRCA			;posun bity v registru - vysila se z D7
        LD	C,A		;vysilany byte zpatky
        LD	(SERIAL),A	;vysli bit na sbernici
        DEC	B		;pocitadlo odeslanych bitu
        LD	A,A
        JP	NZ,L0015	;neni-li B=0, zpracuj dalsi bit
        PUSH	BC		;casova prodleva
        POP	BC
        LD	A,(V4251)
        LD	A,C		;vysilany byte
        OR	A		;nastav FLAGS (CY=0)
        JP	PE,L0016	;sudy pocet bitu, skoc
        SCF			;nastav CY
L0016:  RRA			;CY do D7
        LD	(SERIAL),A	;vystav na sbernici
        SUB	A		;nuluj A
        LD	(V4251),A	;
        CALL	L0009
        LD	A,0FFh		;prednastaveni klavesnice/serioveho kanalu
        LD	(SERIAL),A	;zapis
        JP	L0017		;obnoveni registru, povol preruseni, navrat

L0010:  LD	A,(SERIAL)	;cti seriova data
        RLCA			;D7 do CY
        JP	C,L0011
        LD	A,(V4250)	;byl prijaty ocekavany byte?
        OR	A
        JP	Z,L0017		;obnoveni registru, povol preruseni, navrat
        EX	DE,HL		;schovej si HL
        LD	HL,(V4240)	;adresa v bufferu pro zapis prijatych dat
        LD	(HL),B		;uloz prijaty byte do bufferu
        INC	L		;dalsi adresa v bufferu - max. 256 byte
        LD	(V4240),HL	;vrat adresu na misto
        EX	DE,HL		;obnov si HL
        POP	DE		;obnov registry
        POP	BC
        POP	AF
        EI			;povol preruseni
        RET			;navrat

L0008:  CALL	L0009		;nejaky odpocet
L0011:  LD	A,080h
        LD	(V425B),A
        JP	L0017		;obnoveni registru, povol preruseni, navrat

;*******************************************************************************
;Casova prodleva s delkou v DE
;*******************************************************************************
L0005:	DEC	DE
        LD	A,D
        OR	E
        JP	NZ,L0005
        RET

L0009:  LD	A,(V4252)
        OR	A
        JP	Z,L0018
        DEC	A
L0018:  LD	(V4252),A
        RET

;*******************************************************************************
;Uklada 16x obsah Acc na adresy od (HL)
;*******************************************************************************
L0001:  LD	D,010h		;pocet kopirovanych byte
L0019:  LD	(HL),A		;uloz Acc na (HL)
        INC	HL		;dalsi adresa v RAM
        DEC	D		;pocitadlo pruchodu
        JP	NZ,L0019	;neni-li 0, dalsi byte
        RET			;navrat

;*******************************************************************************
;Hlavni programova smycka
;*******************************************************************************
L0003:  EI			;povol INTERRUPT
        LD	A,001h
        LD	(V4255),A
L0044:  CALL	L0009
        LD	HL,(V4242)	;adresa pro cteni prijatych dat 
        LD	A,(V4240)	;dolni byte adresy pro zapis prijatych dat
        CP	L		;shoda adresy zapisu a cteni
        JP	Z,L0020		;je shoda, skoc
        LD	A,(HL)		;cti prijaty byte z bufferu
        LD	C,A		;schovej do C
        INC	L		;dalsi adresa prijatych dat
        LD	(V4242),HL	;vrat zpatky na misto
        CP	020h
        JP	NC,L0021
        CP	00Eh
        JP	Z,L0022
        CP	00Fh
        JP	Z,L0023
        CP	011h
        JP	Z,L0024
        CP	012h
        JP	Z,L0025
        CP	013h
        JP	Z,L0026
        CP	014h
        JP	Z,L0027
        CP	00Ah
        JP	Z,L0028
        CP	008h
        JP	Z,L0029
        CP	002h
        JP	NC,L0030
        LD	A,020h		;' '
        LD	(V4000),A	;1. pozice displeje
L0030:  LD	A,(V425E)
        OR	A
        JP	Z,L0020
        LD	A,C		;byte precteny z bufferu prijatych dat
        CP	000h
        JP	Z,L0031
        CP	001h
        JP	Z,L0032
        CP	002h
        JP	Z,L0031
        CP	003h
        JP	Z,L0032
        JP	L0020

;******************************************************************************
;Obsluha kodu 000h a 002h
;******************************************************************************
L0031:  LD	A,(V425E)
        CP	07Fh
        JP	Z,L0033
        LD	HL,(V424A)
        INC	L
        JP	NZ,L0034
        DEC	L
L0034:  LD	(V424A),HL
        LD	A,(V4248)
        CP	L
        JP	Z,L0035
        JP	L0033

;******************************************************************************
;Obsluha kodu 001h a 003h
;******************************************************************************
L0032:  LD	HL,(V424A)
        LD	(V4248),HL
        JP	L0036

;******************************************************************************
;Obsluha kodu 00Fh
;******************************************************************************
L0023:  SUB	A
;******************************************************************************
;Obsluha kodu 00Eh
;******************************************************************************
L0022:  LD	(V4257),A
        JP	L0020

;******************************************************************************
;Obsluha kodu 011h
;******************************************************************************
L0024:  SUB	A
        LD	(V425C),A
        LD	(V425F),A
        JP	L0037

;******************************************************************************
;Obsluha kodu 012h
;******************************************************************************
L0025:  LD	A,0FFh
L0037:  LD	(V425D),A
        JP	L0020

;******************************************************************************
;Obsluha kodu 013h
;******************************************************************************
L0026:  LD	A,0FFh
        LD	(V425C),A
        JP	L0020

;******************************************************************************
;Obsluha kodu 014h
;******************************************************************************
L0027:  LD	A,0FFh
        LD	(V425F),A
        JP	L0020
;******************************************************************************
;Obsluha kodu 020h
;******************************************************************************
L0021:  LD	HL,(V4246)	;kam se do displeje zapise znak z C
        LD	(HL),C		;zapis do znakove VRAM
        LD	A,L		;nizsi byte adresy znakove VRAM
        CP	0FFh		;
        JP	Z,L0038
        INC	HL		;dalsi pozice ve znakove VRAM
L0039:  LD	(HL),020h	;vloz ' ' do znakove VRAM
L0038:  LD	(V4246),HL	;nastav vsechny ukazatele znakove VRAM
        LD	(V4248),HL
        LD	(V424A),HL
        JP	L0036		;

;******************************************************************************
;Obsluha kodu 008h - BACKSPACE
;******************************************************************************
L0029:  LD	HL,(V4246)	;ukazatel ve znakove VRAM
        SUB	A		;nuluj Acc
        CP	L		;porovnej s nizsim byte adresy
        JP	Z,L0036		;v L je jiz 0, skoc
        DEC	HL		;posun na predchozu pozici displeje
        JP	L0039		;na (HL) vlozi ' ', nastavi ukazatele VRAM

;******************************************************************************
;Obsluha kodu 00Ah - pravdepodobne SOFT RESET
;******************************************************************************
L0028:  LD	HL,04000h	;pocatek RAM - znakova VRAM
        LD	(V424A),HL
        LD	(V4248),HL	;ukazatel ve znakove VRAM
        LD	(V4246),HL
        LD	A,020h		;znak ' '
        LD	D,008h		;8 pozic displeje
        CALL	L0019		;presun A na (HL) v poctu D - smazani displeje
L0036:  CALL	L0002		;naplneni VRAM bitmapami jednotlivych znaku
L0035:  SUB	A		;nuluj Acc
        LD	(V4258),A	;nuluj V4258
L0033:  SUB	A
        LD	(V4251),A
L0020:  LD	A,(V4258)
        OR	A
        JP	Z,L0040
        LD	A,(V4251)
        OR	A
        JP	NZ,L0040
        LD	HL,(V424A)	;znakova VRAM
        LD	A,(HL)
        LD	(V4251),A
L0040:  LD	A,(V4252)
        OR	A
        JP	NZ,L0041
        CALL	L0042
L0041:  LD	A,044h		;delka casove smycky
L0043:  DEC	A		;casova smycka
        JP	NZ,L0043
        JP	L0044		;navrat do hlavni programove smycky

;******************************************************************************	
;prenos VRAM na adresy 0C000h-0C007h, cteni klavesnice
;******************************************************************************
L0042:  PUSH	HL		;schovej si registry
        PUSH	DE
        PUSH	BC
        LD	HL,(V4244)	;odkud se budou brat data pro displej
        LD	A,L		;vypocet poradi uradku RA1..RA7 v ramci cislicovek
        RRCA
        RRCA
        RRCA
        CPL
        INC	A
        OR	020h		;povoleni dekoderu MH3205 pro uradky znaku
        LD	C,A		;schovej si vysledek
        LD	DE,DISP-1	;adresa pro zapis do displeje-1 (0C000h az 0C007h)
L0045:  INC	DE		;dalsi pozice displeje pro zobrazeni
        LD	A,(HL)		;byte z VRAM
        EX	DE,HL		;prohod adresy
        LD	(HL),A		;byte do displeje - mikroradek 5 bitu, A0-A2 vybira cislicovku
        EX	DE,HL		;prohod adresy
        DEC	HL		;posun ve VRAM smerem k pocatku
        LD	A,E		;preneseni na adresy 0C000h az 0C007h
        CP	007h		;test na konec displeje
        JP	NZ,L0045	;dalsi cislicovka
        LD	A,C		;poradi uradku RA1..RA7
        LD	(L0000),A	;povol dekoder MH3205 a naadresuj zobrazovany uradek
        LD	A,(V4253)
        OR	A
        JP	NZ,L0046
        LD	A,(SERIAL)	;cteni KBD - neaktivni piny jsou v log. 1
        CPL			;invertuj prectena data
        AND	01Fh		;ponech jen 5 nejnizsich bitu od KBD
        JP	Z,L0046		;zadna klavesa nestisknuta, skoc
        LD	B,0BFh
L0047:  INC	B		;v B bude C0 az C4
        RRCA			;hleda se stisknuty radek
        JP	NC,L0047	;radek neni v CY, dalsi rotace
        LD	A,C		;poradi uradku RA1..RA7 a sloupec klavesnice
        AND	007h		;odmaskuj jen sloupec klavesnice
        RLCA			;posun v ramci byte
        RLCA
        RLCA
        OR	B		;slouceni s konvertovanym radkem
        LD	B,A		;zpatky do B
        LD	(V4253),A	;nastav precteny znak
L0046:  LD	A,L		;nizsi byte adresy VRAM
        CP	0FFh		;test na podteceni VRAM
        LD	A,004h
        JP	NZ,L0048
        LD	HL,0423Fh	;konec VRAM
        SUB	A
L0048:  LD	(V4252),A
        LD	(V4244),HL	;ukazatel ve VRAM
        JP	NZ,L0049
        LD	A,(V4255)
        DEC	A
        LD	(V4255),A
        JP	NZ,L0050
        LD	A,(V4256)
        OR	A
        JP	NZ,L0051
        LD	A,007h
        LD	(V4255),A
        LD	A,0FFh
        LD	(V4256),A
        PUSH	HL
        LD	HL,(V4248)
        LD	D,(HL)
        LD	A,(V4257)
        OR	A
        LD	(HL),00Fh
        JP	Z,L0052
        LD	(HL),00Eh
L0052:  CALL	L0002		;naplneni VRAM bitmapami jednotlivych znaku
        LD	(HL),D
        POP	HL
        JP	L0050

L0051:  LD	A,00Fh		;do Acc 00Fh
        LD	(V4255),A	;dej do (V4255)
        SUB	A		;nuluj Acc
        LD	(V4256),A	;nuluj (V4256)
        CALL	L0002		;naplneni VRAM bitmapami jednotlivych znaku
L0050:  LD	A,(V4253)
        LD	B,A
        LD	A,(V4254)
        CP	B
        JP	Z,L0053		;nulovani Acc a V4253, obnoveni registru, navrat
        LD	A,(V4259)
        OR	A
        JP	NZ,L0054
        LD	A,002h
        LD	(V4259),A
        LD	A,B
        LD	(V4254),A
        OR	A
        JP	Z,L0053		;nulovani Acc a V4253, obnoveni registru, navrat
        LD	A,(V4257)
        OR	A
        JP	Z,L0055
        LD	L,B
        LD	H,005h
        JP	L0056

;*******************************************************************************
;Dekodovani stisknute klavesy
;*******************************************************************************
L0055:  LD	A,B
        RLCA
        RLCA
        LD	H,003h
        LD	L,A
        ADD	HL,HL
        INC	HL		;spoctena adresa 8. byte v generatoru znaku
L0056:  LD	A,(HL)		;cti byte z tabulky
        LD	C,A		;zkopiruj precteny znak
        CP	009h		;byte u 'D'
        JP	Z,L0057
        CP	008h		;byte u '<'
        JP	Z,L0058
        CP	00Eh		;byte u 'A'
        JP	Z,L0059
        CP	00Fh		;byte u '9'
        JP	Z,L0060
        CP	003h		;byte u '@'
        JP	Z,L0061
        CP	07Fh		;byte u 'L'
        JP	Z,L0062
        LD	A,(V425F)
        OR	A
        JP	NZ,L0063
        LD	A,C		;kod klavesy precteny z tabulky vedle gen. znaku
        CP	00Ah		;byte u '8'
        JP	Z,L0064
        CP	00Dh		;byte u 'P'
        JP	Z,L0065
        CALL	L0066
        LD	A,(V425C)
        OR	A
        JP	NZ,L0067
        JP	L0053		;nulovani Acc a V4253, obnoveni registru, navrat

L0066:  LD	HL,(V424A)	;adresa ve znakove VRAM
        EX	DE,HL
        LD	HL,(V4248)	;adresa ve znakove VRAM
        LD	A,L		;porovnej pozici na displeji
        CP	E
        JP	C,L0068
        LD	(HL),C		;znak k zobrazeni do znakove VRAM
L0068:  LD	A,(V4246)
        CP	L
        JP	NZ,L0069
        CP	0FFh
        JP	Z,L0070
        INC	L
        LD	(HL),020h	;vlozeni ' ' do znakove VRAM
        INC	A
        LD	(V4246),A
        JP	L0071

L0069:  INC	L
L0071:  LD	(V4248),HL
L0070:  CALL	L0002		;naplneni VRAM bitmapami jednotlivych znaku
        RET

L0062:  LD	HL,(V4246)
        LD	A,(V424A)
        CP	L
        JP	Z,L0063
        DEC	L
        LD	(V4246),HL
        LD	(V4248),HL
        LD	(HL),020h
        CALL	L0002		;naplneni VRAM bitmapami jednotlivych znaku
        JP	L0053		;nulovani Acc a V4253, obnoveni registru, navrat

L0061:  LD	HL,04000h
        LD	(V4248),HL
        CALL	L0002		;naplneni VRAM bitmapami jednotlivych znaku
        JP	L0053		;nulovani Acc a V4253, obnoveni registru, navrat

L0057:  LD	HL,(V4248)	;ukazatel ve znakove VRAM
        SUB	A		;nuluj Acc
        CP	L		;je ukazatel na zacatku?
        JP	Z,L0053		;je, skoc
        DEC	L		;sniz ukazatel ve znakove VRAM
        LD	(V4248),HL	;vrat zpatky na misto
        CALL	L0002		;naplneni VRAM bitmapami jednotlivych znaku
        JP	L0053		;nulovani Acc a V4253, obnoveni registru, navrat

L0058:  LD	HL,(V4248)	;ukazatel ve znakove VRAM
        LD	A,(V4246)	;LOW byte ukazatele ve znakove VRAM
        CP	L		;porovnej ukazatele
        JP	Z,L0053		;nulovani Acc a V4253, obnoveni registru, navrat
        INC	L		;zvetsi ukazatel
        LD	(V4248),HL	;vrat na misto
        CALL	L0002		;naplneni VRAM bitmapami jednotlivych znaku
        JP	L0053		;nulovani Acc a V4253, obnoveni registru, navrat

L0059:  LD	A,0FFh
        LD	(V4257),A
        JP	L0053		;nulovani Acc a V4253, obnoveni registru, navrat

L0060:  SUB	A
        LD	(V4257),A
        JP	L0053		;nulovani Acc a V4253, obnoveni registru, navrat

L0063:  LD	A,(V4251)
        OR	A
        JP	NZ,L0053	;nulovani Acc a V4253, obnoveni registru, navrat
        LD	A,C		;kod klavesy precteny z tabulky vedle gen. znaku
        LD	(V4251),A
        JP	L0053		;nulovani Acc a V4253, obnoveni registru, navrat

L0054:  DEC	A
        LD	(V4259),A
        JP	L0053		;nulovani Acc a V4253, obnoveni registru, navrat
	
L0073:  CP	0FFh
        JP	Z,L0072
        JP	L0053		;nulovani Acc a V4253, obnoveni registru, navrat

L0064:  LD	A,(V425D)
        OR	A
        JP	NZ,L0067
        LD	C,00Ah
L0065:  CALL	L0066
L0067:  LD	A,(V4258)
        OR	A
        JP	NZ,L0053	;nulovani Acc a V4253, obnoveni registru, navrat
        LD	HL,(V4248)
        LD	A,(V424A)
        CP	L
        JP	Z,L0073
        JP	NC,L0053	;nulovani Acc a V4253, obnoveni registru, navrat
L0072:  LD	A,0FFh
        LD	(V4258),A
L0053:  SUB	A
        LD	(V4253),A
L0049:  POP	BC
        POP	DE
        POP	HL
        RET

;*******************************************************************************
;Vytvari bitovy obraz ve VRAM toho co se ma zobrazovat na displeji
;*******************************************************************************
L0002:  PUSH	BC		;schovej si registry
        PUSH	DE
        PUSH	HL
        LD	DE,0423Fh	;konec VRAM
        LD	HL,(V4248)	;adresa znaku do VRAM
        LD	A,L		;kontrola,aby se zacinalo
        CP	007h		;  na 8. znaku displeje
        JP	NC,L0074
        LD	L,007h
L0074:  PUSH	HL		;adresa zpracovavaneho znaku
        LD	L,(HL)		;cti ASCII kod znaku
        LD	A,L		;zkopiruj do Acc
        CP	01Bh		;test na ESC?
        JP	NZ,L0075
        LD	L,00Bh		;prekoduj znak 01Bh na 00Bh
L0075:  LD	H,000h		;v L je ASCII kod znaku
        ADD	HL,HL		;HL*8 - sirka mapy znaku je 8 byte
        ADD	HL,HL		;pro znak ' ' (V20) vyjde 0100h
        ADD	HL,HL
        LD	BC,00500h	;pripocti pocatek generatoru znaku -0100h
        ADD	HL,BC		;v HL pocatek bitmapy znaku
L0076:  LD	A,(HL)		;cti byte z generatoru znaku
        EX	DE,HL		;
        LD	(HL),A		;uloz bitmapu do VRAM
        EX	DE,HL		;
        INC	HL		;dalsi pozice v generatoru znaku
        LD	A,E		;pozice kam se ukladaji bitmapy
        SUB	008h		;posun se o 8 byte zpatky
        LD	E,A		;  kam se uklada bitmapa znaku
        CP	008h		;test na pocatek VRAM
        JP	NC,L0076	;jeste neni, dalsi uradek znaku
        POP	HL		;ukazatel na ASCII znaku
        DEC	HL		;predchozi znak
        CP	000h		;je uz 0?
        JP	Z,L0077		;ano, skonci
        ADD	A,037h		;posun na posledni pozici znaku ve VRAM
        LD	E,A
        JP	L0074		;zpracuj dalsi znak do VRAM
L0077:  POP	HL		;obnov si registry
        POP	DE
        POP	BC
        RET			;navrat

        DS      0E9h,0FFh       ;233 byte 0FFh
	
	DB	004h,01Fh,004h,01Fh,004h,00Eh,004h,0FFh
	DB	000h,000h,008h,01Fh,008h,000h,000h,0FFh
	DB	0FFh,0FFh,0FFh,0FFh,0FFh,0FFh,0FFh,0FFh
	DB	000h,000h,012h,01Fh,002h,000h,000h,0FFh
	DB	01Fh,000h,000h,000h,000h,000h,000h,0FFh
	DB	000h,000h,000h,000h,000h,000h,01Fh,0FFh

        DS      040h,0FFh       ;64 byte 0FFh

	DB	031h,033h,035h,034h,032h,0FFh,0FFh,0FFh
	DB	02Dh,03Dh,03Ah,02Fh,02Bh,0FFh,0FFh,0FFh
	DB	03Eh,026h,021h,03Bh,022h,0FFh,0FFh,0FFh
	DB	00Ah,00Fh,020h,020h,008h,0FFh,0FFh,0FFh
	DB	003h,00Eh,0FFh,0FFh,009h,0FFh,0FFh,00Fh
	DB	01Bh,02Eh,03Fh,02Ch,025h,0FFh,0FFh,0FFh
	DB	00Dh,028h,024h,02Ah,029h,0FFh,0FFh,0FFh
	DB	030h,038h,036h,037h,039h,0FFh,0FFh

V05FF:  DB      093h            ;identifikacni cislo terminalu?

;*******************************************************************************
;Generator znaku v matici 8x8 bodu, vyuziva se 7 byte a z kazdeho byte 5 bitu.
;1. byte je nejvrchnejsi radek, 7. byte je posledni radek znakovky, LSB bity
;bitmapy jsou vlevo:
;
;napr.	DB	00Fh,011h,011h,00Fh,011h,011h,00Fh,0FFh		;'B'
;bit    01234567
;00Fh	xxxx.
;011h	x...x
;011h	x...x
;00Fh	xxxx.
;011h	x...x
;011h	x...x
;00Fh	xxxx.
;0FFh je pouzito k dekodovani stisknute klavesy
;*******************************************************************************
CHRGEN: DB	000h,000h,000h,000h,000h,000h,000h,051h		;' '
        DB	004h,004h,004h,004h,004h,000h,004h,045h		;'!'
        DB	00Ah,00Ah,00Ah,000h,000h,000h,000h,054h		;'"'
        DB	00Ah,00Ah,01Fh,00Ah,01Fh,00Ah,00Ah,052h		;'#'
        DB	000h,011h,00Eh,00Ah,00Eh,011h,000h,057h		;'$'
        DB	003h,013h,008h,004h,002h,019h,018h,0FFh		;'%'
        DB	004h,00Ah,00Ah,006h,015h,009h,016h,0FFh		;'&'
        DB	004h,004h,002h,000h,000h,000h,000h,0FFh		;'''
        DB	008h,004h,002h,002h,002h,004h,008h,041h		;'('
        DB	002h,004h,008h,008h,008h,004h,002h,044h		;')'
        DB	000h,004h,015h,00Eh,015h,004h,000h,047h		;'*'
        DB	000h,004h,004h,01Fh,004h,004h,000h,046h		;'+'
        DB	000h,000h,000h,000h,002h,002h,001h,053h		;','
        DB	000h,000h,000h,01Fh,000h,000h,000h,0FFh		;'-'
        DB	000h,000h,000h,000h,000h,000h,002h,0FFh		;'.'
        DB	000h,010h,008h,004h,002h,001h,000h,0FFh		;'/'
        DB	00Eh,011h,019h,015h,013h,011h,00Eh,03Ch		;'0'
        DB	004h,006h,004h,004h,004h,004h,00Eh,058h		;'1'
        DB	00Eh,011h,010h,00Ch,002h,001h,01Fh,056h		;'2'
        DB	01Fh,010h,008h,00Ch,010h,011h,00Eh,043h		;'3'
	DB	008h,00Ch,00Ah,009h,01Fh,008h,008h,05Ah		;'4'
        DB	01Fh,001h,00Fh,010h,010h,011h,00Eh,0FFh		;'5'
        DB	01Ch,002h,001h,00Fh,011h,011h,00Eh,0FFh		;'6'
        DB	01Fh,010h,008h,004h,002h,002h,002h,0FFh		;'7'
        DB	00Eh,011h,011h,00Eh,011h,011h,00Eh,00Ah		;'8'
        DB	00Eh,011h,011h,01Eh,010h,008h,007h,00Fh		;'9'
        DB	000h,000h,000h,000h,002h,000h,002h,020h		;':'
        DB	000h,000h,004h,000h,004h,004h,002h,020h		;';'
        DB	008h,004h,002h,001h,002h,004h,008h,008h		;'<'
        DB	000h,000h,01Fh,000h,01Fh,000h,000h,0FFh		;'='
        DB	002h,004h,008h,010h,008h,004h,002h,0FFh		;'>'
        DB	00Eh,011h,010h,008h,004h,000h,004h,0FFh		;'?'
        DB	006h,011h,019h,015h,01Dh,001h,00Eh,003h		;'@'
        DB	004h,00Ah,011h,011h,01Fh,011h,011h,00Eh		;'A'
        DB	00Fh,011h,011h,00Fh,011h,011h,00Fh,0FFh		;'B'
        DB	00Eh,011h,001h,001h,001h,011h,00Eh,0FFh		;'C'
        DB	00Fh,012h,012h,012h,012h,012h,00Fh,009h		;'D'
        DB	01Fh,001h,001h,00Fh,001h,001h,01Fh,0FFh		;'E'
        DB	01Fh,001h,001h,001h,001h,001h,001h,0FFh		;'F'
        DB	00Eh,011h,001h,001h,019h,011h,01Eh,0FFh		;'G'
        DB	011h,011h,011h,01Fh,011h,011h,01Fh,01Bh		;'H'
        DB	00Eh,004h,004h,004h,004h,004h,00Eh,04Dh		;'I'
        DB	010h,010h,010h,010h,011h,111h,00Eh,042h		;'J'
        DB	011h,009h,005h,003h,005h,009h,011h,04Eh		;'K'
        DB	001h,001h,001h,001h,001h,001h,01Fh,07Fh		;'L'
        DB	011h,01Bh,015h,015h,011h,011h,011h,0FFh		;'M'
        DB	011h,011h,013h,015h,019h,011h,011h,0FFh		;'N'
        DB	00Eh,011h,011h,011h,011h,011h,00Eh,0FFh		;'O'
        DB	00Fh,011h,011h,00Fh,001h,001h,001h,00Dh		;'P'
        DB	00Eh,011h,011h,011h,015h,009h,016h,04Bh		;'Q'
        DB	00Fh,011h,011h,00Fh,005h,009h,011h,048h		;'R'
        DB	00Eh,011h,011h,00Eh,010h,011h,00Eh,04Ah		;'S'
        DB	01Fh,004h,004h,004h,004h,004h,004h,04Ch		;'T'
        DB	011h,011h,011h,011h,011h,011h,00Eh,0FFh		;'U'
        DB	011h,011h,011h,00Ah,00Ah,004h,004h,0FFh		;'V'
        DB	011h,011h,011h,015h,015h,015h,00Ah,0FFh		;'W'
        DB	011h,011h,00Ah,004h,00Ah,011h,011h,050h		;'X'
        DB	011h,011h,004h,004h,004h,004h,004h,049h		;'Y'
        DB	01Fh,010h,008h,004h,002h,001h,01Fh,059h		;'Z'
        DB	007h,001h,001h,001h,001h,001h,007h,055h		;'['
        DB	000h,001h,002h,004h,008h,010h,000h,04Fh		;'\'
        DB	01Ch,010h,010h,010h,010h,010h,01Ch,0FFh		;']'
        DB	00Eh,011h,000h,000h,000h,000h,000h,0FFh		;'^'
        DB	000h,000h,000h,000h,000h,000h,011h,04Fh		;'_'


	ORG	RAM_ST
V4000:	DB	000h,000h,000h,000h,000h,000h,000h,000h		;1. az 8. pozice displeje
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	
	ORG	04100h
V4100:	DS	0FFh                    ;buffer znaku prijatych pres seriove rozhrani

	ORG	04200h
V4200:	DB	000h,000h,000h,000h,000h,000h,000h,000h
V4208:	DB	000h,000h,000h,000h,000h,000h,000h,000h		;bitmapa toho, co se ma
	DB	000h,000h,000h,000h,000h,000h,000h,000h		;  zobrazovat na displeji
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h

V4240:	DW	00000h		;04100h - adresa bufferu pro ukladani prijatych dat
V4242:	DW	00000h		;04100h - adresa bufferu pro cteni prijatych dat
V4244:	DW	00000h		;0423Fh - posledni adresa VRAM
V4246:	DW	00000h		;04000h - adresy ukazatelu ve znakove VRAM
V4248:	DW	00000h		;04000h
V424A:	DW	00000h		;04000h

	ORG	RAM_VR
V4250:	DB	000h
V4251:	DB	000h
V4252:	DB	000h
V4253:	DB	000h		;precteny znak z klavesnice
V4254:	DB	000h
V4255:	DB	000h		;citac pruchodu - 001h,007h nebo 00Fh
V4256:	DB	000h
V4257:	DB	000h
V4258:	DB	000h
V4259:	DW	00000h
V425B:	DB	000h
V425C:	DB	000h
V425D:	DB	000h
V425E:	DB	000h
V425F:  DB      000h

;Pripojeni klavesnice TS 522 0001:
;
;MH3205	 Y0	 Y1	 Y2	 Y3	 Y4	 Y5	 Y6	 Y7
;---+---+-------+-------+-------+-------+-------+-------+-------+-------+
;D0 | 6 |   Q   |   A   |   <   |   v   |  |<-  |   ->  | ENTER |   P   |
;---+---+-------+-------+-------+-------+-------+-------+-------+-------+
;D1 | 7 |   E   |   D   |   X   |   ↓   |   ^   |   M   |   K   |   I   |
;---+---+-------+-------+-------+-------+-------+-------+-------+-------+
;D2 | 8 |   T   |   G   |   V   |       |       |   B   |   H   |   Y   |
;---+---+-------+-------+-------+-------+-------+-------+-------+-------+
;D3 | 9 |   R   |       |   C   |  SPC  |       |   N   |   J   |   U   |
;---+---+-------+-------+-------+-------+-------+-------+-------+-------+
;D4 |10 |   W   |   S   |   Z   |    >  |   <   |  DEL  |   L   |   O   |
;---+---+-------+-------+-------+-------+-------+-------+-------+-------+
;  	   1	   2	   3     4,11,12    5	    13	    14	    15
